import AlteryxPythonSDK as Sdk
import connectors_core.alteryx as alteryx
import onedrivesdk
import requests
from io import BytesIO
import json
from connectors_core.file import DelimitedReader, JsonPivotInputReader, AvroInputReader, FileFormat
from onedrivesdk import error
from connectors_core.onedrive.od import OneDriveAuthProvider
from connectors_core.file.errors import DelimitedIndexError
from onedrivesdk.request.item_request_builder import ItemRequestBuilder
from .common import (API_BASE_URL)

from typing import Union


class OneDriveInputEngine(alteryx.AyxPluginBase):

    def __init__(self, n_tool_id: int, alteryx_engine: Sdk.AlteryxEngine, output_anchor_mgr: Sdk.OutputAnchorManager): 
        super().__init__(n_tool_id, alteryx_engine, output_anchor_mgr)

        self.od_client_id = None
        self.od_file_name = None
        self.od_refresh_token = None
        self.od_access_token = None
        self.od_secret_key = None
        self.od_expiration = None
        self.od_token_response = None
        self.od_scopes = None
        self.od_file_format = FileFormat.Delimited
        self.od_delimited_has_header = True
        self.od_field_delimiter = ','
        self.od_field_quote_character = '"'
        self.od_encoding = 'utf-8'
        self.od_json_force_string_values = True
        self.od_start_import_at_line = 1
        self.od_max_field_length = None
        self.od_authentication_type = None

        # One Drive Auth Configs
        self.client = None
        self.auth_provider = None
        self.auth_url = "https://login.microsoftonline.com/common/oauth2/v2.0/authorize"
        self.token_url = "https://login.microsoftonline.com/common/oauth2/v2.0/token"
        self.redirect_uri = 'https://login.live.com/oauth20_desktop.srf'

    def pi_init(self, str_xml: str):
        super().pi_init(str_xml)

        self.od_client_id = self.get_xml_config_value(str_xml, 'od_ClientId')
        self.od_file_name = self.get_xml_config_value(str_xml, 'od_SelectedFile')
        self.od_authentication_type = self.get_xml_config_value(str_xml, 'od_AccountType')

        if self.od_authentication_type == 'Business':
            self.od_secret_key = self.get_encrypted_xml_config_value(str_xml, 'od_ClientSecret')

        token_response = self.get_encrypted_xml_config_value(str_xml, 'od_AuthToken')

        if not self.od_client_id:
            self.invalidate_config(self.alteryx_engine.xmsg("Please enter a Client ID"))
        if not self.od_secret_key and self.od_authentication_type == 'Business':
            self.invalidate_config(self.alteryx_engine.xmsg("Please enter a Client secret"))

        if not token_response:
            self.invalidate_config(self.alteryx_engine.xmsg('Please Log in'))
        else:
            try:
                token_response = json.loads(token_response)
                self.od_access_token = token_response["access_token"]
                self.od_refresh_token = token_response["refresh_token"]
                self.od_expiration = token_response["expires_at"]
                self.od_scopes = token_response["scope"]
            except Exception as e:
                self.invalidate_config(self.alteryx_engine.xmsg("Failed parsing authentication: @1", str(e)))

        self.od_file_format = FileFormat[
            self.get_xml_config_value(str_xml, 'od_FileFormat') or FileFormat.Unknown.name
        ]

        self.od_delimited_has_header = (self.get_xml_config_value(str_xml, 'od_DelimitedHasHeader') == "True")
        self.od_start_import_at_line = int(self.get_xml_config_value(str_xml, 'od_StartImportAtLine') or 1)
        self.od_json_force_string_values = (self.get_xml_config_value(str_xml, 'od_JSONForceStringValues') == "True")
        self.od_max_field_length = int(self.get_xml_config_value(str_xml, 'od_MaxFieldLength') or 255)

        if not self.od_file_name:
            self.invalidate_config(self.alteryx_engine.xmsg('Please select a file'))

        try:
            if self.od_start_import_at_line < 1:
                self.invalidate_config(self.alteryx_engine.xmsg("Starting line must be greater than 0"))
        except ValueError:
            self.invalidate_config(self.alteryx_engine.xmsg("Starting line must be a number"))
        
        try: 
            self.od_field_quote_character = {
                'QUOTE': '\'',
                'DOUBLE_QUOTE': '"',
                'BACKTICK': '`'
            }[self.get_xml_config_value(str_xml, 'od_QuoteCharacter') or 'DOUBLE_QUOTE']
        except KeyError:
            self.invalidate_config(self.alteryx_engine.xmsg("Invalid Quote character"))

        self.od_encoding = self.get_xml_config_value(str_xml, 'od_CodePage') or 'utf-8'

        try:
            self.od_field_delimiter = {
                'COMMA': ',',
                'TAB': '\t',
                'SPACE': ' ',
                'PIPE': '|'
            }[self.get_xml_config_value(str_xml, 'od_Delimiter') or 'COMMA']
        except KeyError:
            self.invalidate_config(self.alteryx_engine.xmsg("Invalid Field delimiter"))

        self.output_anchor = self.output_anchor_mgr.get_output_anchor('Output')

        metadata_dict = {
            'Host': API_BASE_URL,
            'ServiceName': 'OneDrive',
            'PluginName': 'OneDrive'
        }

        self.alteryx_engine.create_connect_metadata(self.n_tool_id, metadata_dict)

        try:
            if self.od_access_token:
                http_provider = onedrivesdk.HttpProvider()
                self.auth_provider = OneDriveAuthProvider(
                    http_provider=http_provider,
                    client_id=self.od_client_id,
                    scopes=self.od_scopes,
                    client_secret=self.od_secret_key,
                    expires_in=self.od_expiration,
                    access_token=self.od_access_token,
                    refresh_token=self.od_refresh_token,
                    redirect_uri=self.redirect_uri,
                    alteryx_engine=self.alteryx_engine
                )

                self.auth_provider.refresh_token()

                self.client = onedrivesdk.OneDriveClient(API_BASE_URL, self.auth_provider, http_provider)
        except error.OneDriveError as e:
            self.display_error_msg(self.alteryx_engine.xmsg(
                "There was a problem authenticating with OneDrive: @1", str(e)
            ))
            return False

    def pi_add_incoming_connection(self, str_type: str, str_name: str) -> Union[alteryx.IncomingConnectionBase, None]:
        
        self.display_error_msg(self.alteryx_engine.xmsg("This tool does not accept incoming connections"))

        return None

    def pi_add_outgoing_connection(self, str_name: str) -> bool:
        return True

    def pi_push_all_records(self, n_record_limit: int = -1) -> bool:
        if not self.is_initialized:
            return False

        update_only = self.alteryx_engine.get_init_var(self.n_tool_id, 'UpdateOnly') == 'True' \
                      or n_record_limit == 0

        #path type file input

        tmp = self.od_file_name[12:]

        path = ':/' + tmp + ":" if tmp.count('/') > 1 else '/' + tmp

        try:
            if self.od_authentication_type == 'Personal':
                request = self.client.drives["me"].items["root"].children[path].request()
            else:
                if tmp.count('/') >= 1:
                    request_url = 'https://graph.microsoft.com/v1.0/me/drive/root' + path
                else:
                    request_url = 'https://graph.microsoft.com/v1.0/me/drive/root/children' + path
                request = ItemRequestBuilder(request_url, self.client).request()

            self.auth_provider.authenticate_request(request)

            size = request.get().size

            if tmp.count('/') > 1:
                requesturl = 'https://graph.microsoft.com/v1.0/me/drive/root' + path + '/content'
            else:
                requesturl = 'https://graph.microsoft.com/v1.0/me/drive/root/children' + path + '/content'

            response = requests.get(
                requesturl,
                stream=True,
                headers=request._headers
            )
        except error.OneDriveError as e:
            self.log.exception(e)
            self.display_error_msg(self.alteryx_engine.xmsg("There was an error retrieving file from OneDrive"))
            return False

        try:
            with BytesIO(response.content) as input_stream:
                if self.od_file_format == FileFormat.JSON:
                    reader = JsonPivotInputReader(
                        input_stream,
                        self.output_anchor,
                        self.alteryx_engine,
                        size,
                        self.get_field_source,
                        force_string_values=self.od_json_force_string_values,
                        max_field_size=self.od_max_field_length
                    )
                elif self.od_file_format == FileFormat.Delimited:
                    reader = DelimitedReader(
                        input_stream,
                        self.output_anchor,
                        self.alteryx_engine,
                        size,
                        self.get_field_source,
                        binary_input=True,
                        max_field_size=self.od_max_field_length,
                        delimiter=self.od_field_delimiter,
                        quotechar=self.od_field_quote_character,
                        encoding=self.od_encoding,
                        has_header_row=self.od_delimited_has_header,
                        start_data_at_line=self.od_start_import_at_line
                    ) 
                elif self.od_file_format == FileFormat.Avro:
                    reader = AvroInputReader(
                        input_stream,
                        self.output_anchor,
                        self.alteryx_engine,
                        size,
                        self.get_field_source
                    )
                else:
                    self.display_error_msg(self.alteryx_engine.xmsg("Unknown File format!"))
                    return False
           
                reader.init()
                if update_only:
                    return True

                record_count = reader.read(n_record_limit)
            
            self.display_info_msg(self.alteryx_engine.xmsg(
                "@1 records were read from https://graph.microsoft.com/v1.0/@2", str(record_count), self.od_file_name
            ))
            self.output_anchor.update_progress(1.0)
            self.output_anchor.close()
            return True
        except DelimitedIndexError as e:
            self.display_error_msg(self.alteryx_engine.xmsg(
                "There was an error processing your file. Check your quote character configuration"
            ))
            self.log.exception("There was a problem processing your file. Check your quote character: {}".format(e))
            return False
        except BaseException as e:
            self.log.exception(e)
            self.display_error_msg(self.alteryx_engine.xmsg(
                "There was problem processing your file: @1", str(e)
            ))
            return False

    def report_input_stream_progress(self, stream: BytesIO, stream_length):
        stream_progress = round(stream.tell() / stream_length, 2)
        if (stream_progress * 100) % 5 == 0:
            self.output_anchor.update_progress(stream_progress)

    def pi_close(self, b_has_errors: bool):
        super().pi_close(b_has_errors)
        self.output_anchor.assert_close()

    def get_field_source(self, field: Sdk.Field):
        return "OD: https://graph.microsoft.com/v1.0/{}".format(
            self.od_file_name
        )
