import AlteryxPythonSDK as Sdk
from connectors_core import alteryx
import onedrivesdk
from onedrivesdk import error
from typing import Union, Any
from connectors_core.onedrive.od import OneDriveAuthProvider
from connectors_core.onedrive.filewriters import ODDelimitedFileWriter, ODAvroFileWriter, ODJSONFileWriter
from connectors_core.file import FileFormat, FileExistsBehavior
from connectors_core.file.errors import FileOpenError, FileWriterAbortedError
from .common import (API_BASE_URL)


class OneDriveIncomingConnectionHandler(alteryx.IncomingConnectionBase):

    def __init__(self, parent: alteryx.AyxPluginBase):
        super().__init__(parent)
        self.writer = None
        self.record_info = None  # type: Sdk.RecordInfo
        self.is_initialized = False
        self.auth_provider = None
        self.client = None

    def ii_init(self, record_info_in: Sdk.RecordInfo) -> bool:
        if not self.parent.is_initialized:
            return False
        try:
            http_provider = onedrivesdk.HttpProvider()
            self.auth_provider = OneDriveAuthProvider(
                http_provider=http_provider,
                client_id=self.parent.client_id,
                scopes=self.parent.scopes,
                client_secret=self.parent.client_secret,
                expires_in=self.parent.expiration,
                access_token=self.parent.access_token,
                refresh_token=self.parent.refresh_token,
                redirect_uri=self.parent.redirect_uri,
                alteryx_engine=self.parent.alteryx_engine
            )
            self.auth_provider.refresh_token()
            self.client = onedrivesdk.OneDriveClient(API_BASE_URL, self.auth_provider, http_provider)

            def conversion_error_handler(field: Sdk.Field, record: Sdk.RecordRef, e: BaseException) -> Any:
                if field.get_as_string(record) is None:
                    self.parent.display_warning_msg(self.parent.alteryx_engine.xmsg(
                            "Missing value for field @1 (@2): @3", field.name, str(field.type), str(e)
                    ))
                else:
                    self.parent.display_error_msg(self.parent.alteryx_engine.xmsg(
                            "Data conversion error: encountered for field @1 (@2): @3", field.name, str(field.type), str(e)
                    ))
                return None
            temp = self.parent.alteryx_engine.create_temp_file_name()

            if self.parent.file_format == FileFormat.Delimited:
                self.writer = ODDelimitedFileWriter(
                    client=self.client,
                    file_path=self.parent.file_path,
                    temp_file=temp,
                    write_header=self.parent.delimited_has_header,
                    data_conversion_error_handler=conversion_error_handler,
                    file_exists_behavior=self.parent.existing_file_behavior,
                    alteryx_engine=self.parent.alteryx_engine,
                    encoding=self.parent.encoding
                )
            elif self.parent.file_format == FileFormat.Avro:
                self.writer = ODAvroFileWriter(
                    client=self.client,
                    file_path=self.parent.file_path,
                    temp_file=temp,
                    data_conversion_error_handler=conversion_error_handler,
                    file_exists_behavior=self.parent.existing_file_behavior,
                    alteryx_engine=self.parent.alteryx_engine
                )
            elif self.parent.file_format == FileFormat.JSON:
                self.writer = ODJSONFileWriter(
                    client=self.client,
                    file_path=self.parent.file_path,
                    temp_file=temp,
                    data_conversion_error_handler=conversion_error_handler,
                    file_exists_behavior=self.parent.existing_file_behavior,
                    alteryx_engine=self.parent.alteryx_engine
                )
            else:
                self.is_initialized = False
                return False
            self.writer.init(record_info_in)
            self.is_initialized = True
            return True
        except error.OneDriveError as e:
            self.is_initialized = False
            self.parent.display_error_msg(self.parent.alteryx_engine.xmsg(
                "There was a problem authentication with Onedrive: @1", str(e)
            ))
        except ConnectionError as e:
            self.is_initialized = False
            self.parent.display_error_msg(self.parent.alteryx_engine.xmsg(
                "Failed to initialize OneDrive Output: @1", str(e)
            ))
        except Exception as e:
            self.parent.display_error_msg(self.parent.alteryx_engine.xmsg("Failed to make connection: @1", str(e)))

    def ii_push_record(self, in_record: Sdk.RecordRef) -> bool:
        try:
            if not self.writer.is_open():
                self.writer.open()
            return self.writer.write(in_record)
        except FileOpenError as e:
            self.parent.display_error_msg(self.parent.alteryx_engine.xmsg(
                "Failed to open file for writing: @1", str(e)
            ))
            return False
        except FileWriterAbortedError as e:
            self.parent.display_error_msg(self.parent.alteryx_engine.xmsg(
                "The write operation was aborted: @1", str(e)
            ))
            return False

    def ii_update_progress(self, d_percent: float):
        try:
            self.parent.alteryx_engine.output_tool_progress(self.parent.n_tool_id, d_percent / 2.0)
        except AttributeError:
            pass

    def ii_close(self):
        self.is_initialized = False
        try:
            self.writer.close()
            self.parent.alteryx_engine.output_tool_progress(self.parent.n_tool_id, 1.0)
        except error.OneDriveError as e:
            self.log.error('OneDriveError in closing the file writer: %s', e)
            self.parent.display_error_msg(self.parent.alteryx_engine.xmsg(
                "Could not post file to OneDrive please verify your conflict handler"
            ))
            return False
        except Exception as e:
            self.log.error("Error in closing the file writer: %s", e)
            self.parent.display_error_msg(self.parent.alteryx_engine.xmsg("Could not post file"))
            return False
        self.writer = None
