import json
import AlteryxPythonSDK as Sdk
import connectors_core.alteryx as alteryx
from connectors_core.file import FileFormat, FileExistsBehavior
from .common import ( API_BASE_URL )

from typing import Union

from .connections import OneDriveIncomingConnectionHandler


class OneDriveOutputEngine(alteryx.AyxPluginBase):

    def __init__(self, n_tool_id: int, alteryx_engine: Sdk.AlteryxEngine, output_anchor_mgr: Sdk.OutputAnchorManager):
        super().__init__(n_tool_id, alteryx_engine, output_anchor_mgr)
        self.client_id = None  # type: str
        self.account_type = None
        self.client_secret = None  # type: str
        self.file_format = None
        self.file_path = None
        self.existing_file_behavior = None
        self.access_token = None
        self.refresh_token = None
        self.expiration = None
        self.scopes = None
        self.delimited_has_header = True
        self.is_initialized = False
        self.redirect_uri = 'https://login.live.com/oauth20_desktop.srf'
        self.incoming_connection_added = False
        self.encoding = None # type: str

    def pi_add_incoming_connection(self, str_type: str, str_name: str) -> Union[alteryx.IncomingConnectionBase, None]:
        if self.incoming_connection_added:
            self.display_error_msg(self.alteryx_engine.xmsg(
                "Only one incoming connection can be accepted by this tool"
            ))
            return None
        connection_handler = OneDriveIncomingConnectionHandler(self)
        self.incoming_connection_added = True
        return connection_handler

    def pi_init(self, str_xml: str):
        super().pi_init(str_xml)

        self.client_id = self.get_xml_config_value(str_xml, 'od_ClientId')

        self.account_type = self.get_xml_config_value(str_xml, 'od_AccountType')

        if self.account_type == 'Business':
            self.client_secret = self.get_encrypted_xml_config_value(str_xml, 'od_ClientSecret')

        self.file_format = FileFormat[
            self.get_xml_config_value(str_xml, 'od_FileFormat') or FileFormat.Unknown.name
        ]

        self.file_path = self.get_xml_config_value(str_xml, 'od_SelectedFile')

        if self.file_path:
            self.file_path = self.file_path[12:]
        else:
            self.invalidate_config(self.alteryx_engine.xmsg("Please select a file"))
        self.existing_file_behavior = FileExistsBehavior[
            self.get_xml_config_value(str_xml, 'od_ExistingFileBehavior') or FileExistsBehavior.Abort.name
        ]

        if not self.client_id:
            self.invalidate_config(self.alteryx_engine.xmsg("Please provide your Client ID"))

        if not self.client_secret and self.account_type == 'Business':
            self.invalidate_config(self.alteryx_engine.xmsg("Please enter a Client secret"))

        if not self.account_type:
            self.invalidate_config(self.alteryx_engine.xmsg("Please provide your Account type"))

        if self.file_format == FileFormat.Unknown:
            self.invalidate_config(self.alteryx_engine.xmsg("Please specify a File format"))

        token_str = self.get_encrypted_xml_config_value(str_xml, 'od_AuthToken')

        self.encoding = self.get_xml_config_value(str_xml, 'od_CodePage') or 'utf-8'
        if not token_str:
            self.invalidate_config(self.alteryx_engine.xmsg('Please Log in'))
        else:
            try:
                token = json.loads(token_str)
                self.access_token = token["access_token"]
                self.refresh_token = token["refresh_token"]
                self.expiration = token["expires_at"]
                self.scopes = token["scope"]
            except Exception as e:
                self.invalidate_config(self.alteryx_engine.xmsg("Failed parsing authentication: @1", str(e)))

        metadata_dict = {
            'Host': API_BASE_URL,
            'ServiceName': 'OneDrive',
            'PluginName': 'OneDrive'
        }

        self.alteryx_engine.create_connect_metadata(self.n_tool_id, metadata_dict)

        self.output_anchor = self.output_anchor_mgr.get_output_anchor('Output')  # type: Sdk.OutputAnchor

    def pi_add_outgoing_connection(self, str_name: str) -> bool:
        self.display_error_msg(self.alteryx_engine.xmsg('This tool does not allow outgoing connections'))
        return False

    def pi_push_all_records(self, n_record_limit: int) -> bool:
        return False

    def pi_close(self, b_has_errors: bool):
        super().pi_close(b_has_errors)
        if self.output_anchor:
            self.output_anchor.assert_close()
